package f00f.net.napster.japster.download;

import java.io.IOException;

public class SmartDownload implements Runnable
{
	private Download download;
	private DownloadManager manager;
	private RateMonitor rateMonitor;
	private int size = 0;
	private final int fileSize;
	
	public SmartDownload(Download download, int fileSize, DownloadManager manager)
	{
		this.download = download;
		this.manager = manager;
		this.fileSize = fileSize;
		rateMonitor = new RateMonitor(this, manager);
	}
	
	public SmartDownload(Download download, DownloadManager manager)
	{
		this(download, readSize(download), manager);
		// side effect of using readSize()
		++size;
	}
	
	private static int readSize(Download download)
	{
		try
		{
			String size = new String();
			char c;

			do
			{
				while (download.available()==0)
					Thread.yield();

				int x = download.read();
				c = (char)x;

				if (Character.isDigit(c))
					size = size + c;
				else
				{
					download.write(x);
				}

			} while ( Character.isDigit(c) );
			System.out.println("Read file size to be "+size);
			return (new Integer(size)).intValue();
		}
		catch (IOException e)
		{
			e.printStackTrace(System.err);
			return 0;
		}
	}
	
	public int getSize()
	{
		return size;
	}
	
	public void run()
	{
		Thread rateKicker = new Thread(rateMonitor);
		rateKicker.start();
		
		try
		{
			while (size < fileSize && !rateMonitor.isTooSlow())
			{
				int length = readPacket();
				if (length > 0)
				{
					System.out.print("Read "+size+"/"+fileSize+" bytes ");
					System.out.println((size*100/fileSize)+"%");
				}
			}
		}
		catch (IOException e)
		{
			e.printStackTrace(System.out);
		}
		rateMonitor.setDone();
		rateKicker.interrupt();
	}
	
	public boolean download()
	{
		run();
		return (size == fileSize);
	}
	
	private int readPacket()
		throws IOException
	{
		int available = download.available();
		byte b[] = new byte[available];
		int length = download.read(b, 0, available);
		
		download.write(b, 0, length);
		
		size += length;
		return length;
	}

}
