package f00f.net.napster.japster.messages;

import java.io.*;
import java.net.*;
import java.util.*;

import f00f.net.napster.japster.messages.client.ClientMessage;
import f00f.net.napster.japster.messages.server.ServerMessage;

public class MessageQueue implements Enumeration
{
	private InputStream in;
	private OutputStream out;
	private MessageFactory factory;
	
	private LinkedList queue = new LinkedList();
	
	public MessageQueue(InputStream in, OutputStream out,
		MessageFactory factory)
	{
		this.in = in;
		this.out = out;
		this.factory = factory;
	}
	
	public MessageQueue(Socket socket, MessageFactory factory)
		throws IOException
	{
		this(socket.getInputStream(), socket.getOutputStream(), factory);
	}
	
	public Object nextElement()
	{
		if (queue.size()>0)
			return queue.removeFirst();
		else
			return readMessage();
	}
	
	private ServerMessage readMessage()
	{
		try
		{
			short b1 = (short)in.read();
			short b2 = (short)in.read();
			short b3 = (short)in.read();
			short b4 = (short)in.read();
			
			short length = (short)( b1 + (b2<<8) );
			short type = (short)( b3 + (b4<<8) );

			byte body[] = new byte[length];
			
			// make sure we get the whole message
			int total = in.read(body, 0, length);
			while (total < length)
			{
				total += in.read(body, total, length-total);
			}
			String bodyString = new String(body);
			return factory.parseMessage(type, bodyString);
		}
		catch (IOException e)
		{
			e.printStackTrace(System.err);
			throw new NoSuchElementException();
		}
	}
	
	public boolean hasMoreElements()
	{
		try
		{
			return (queue.size()>0) || (in.available() > 0);
		}
		catch (IOException e)
		{
			e.printStackTrace(System.err);
			return (queue.size()>0);
		}
	}
	
	public ServerMessage nextMessage()
	{
		while (!hasMoreElements())
			Thread.yield();
		return (ServerMessage)nextElement();
	}
	
	public boolean sendMessage(ClientMessage message)
	{
		try
		{
			short length = (short)message.getData().length();
			short type = message.getTypeCode();
			char c[] = new char[4];
			c[0] = (char)(length%0xff);
			c[1] = (char)(length/0xff);
			c[2] = (char)(type%0xff);
			c[3] = (char)(type/0xff);

			String data = new String(c) + message.getData();

			System.out.println("-> Sending: '"+message.getData()+"'");
			System.out.println("-> Type: "+Integer.toHexString(type));
			System.out.println("-> Length: "+length);

			out.write( data.getBytes() );
			return true;
		}
		catch (IOException e)
		{
			e.printStackTrace(System.err);
			return false;
		}
	}
	
	public boolean hasMessage(short typeCode)
	{
		ListIterator iterator = queue.listIterator(0);
		while (iterator.hasNext())
		{
			ServerMessage message = (ServerMessage)iterator.next();
			if (message.getTypeCode() == typeCode)
			{
				return true;
			}
		}
		return false;
	}
	
	public ServerMessage nextMessage(short typeCode)
	{
		ListIterator iterator = queue.listIterator(0);
		while (iterator.hasNext())
		{
			ServerMessage message = (ServerMessage)iterator.next();
			if (message.getTypeCode() == typeCode)
			{
				iterator.remove();
				return message;
			}
		}
		return waitForMessage(typeCode);
	}

	public ServerMessage waitForMessage(short typeCode, int waitTime)
	{
		long endTime = System.currentTimeMillis()+waitTime;
		
		ListIterator iterator = queue.listIterator(0);
		while (iterator.hasNext())
		{
			ServerMessage message = (ServerMessage)iterator.next();
			if (message.getTypeCode() == typeCode)
			{
				iterator.remove();
				return message;
			}
		}
		
		do
		{
			try
			{
				if ( in.available() == 0)
					Thread.yield();
				else
				{
					ServerMessage message = readMessage();
					if (message.getTypeCode() == typeCode)
						return message;
					else
					{
//						System.out.println("== Enqueuing message:");
//						System.out.println("== "+message);
						queue.add(message);
					}
				}
			}
			catch (IOException e)
			{
				e.printStackTrace(System.err);
				return null;
			}
		
		} while (System.currentTimeMillis() < endTime || waitTime==0);
		return null;
	}
	
	public ServerMessage waitForMessage(int waitTime)
	{
		long endTime = System.currentTimeMillis()+waitTime;
		do
		{
			try
			{
				if ( in.available() == 0)
					Thread.yield();
				else
				{
					return readMessage();
				}
			}
			catch (IOException e)
			{
				e.printStackTrace(System.err);
				return null;
			}
		
		} while (System.currentTimeMillis() < endTime);
		return null;
	}
}
